//+------------------------------------------------------------------+
//|                                   Copyright  2012, Ivan Kornilov|
//|                                          StochasticStack_v1.3.mq4|
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, Ivan Kornilov. All rights reserved."
#property link "excelf@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_level1 0.1
#property indicator_level2 0.9

#property indicator_color8 Gray  
#property indicator_color7 Gray
#property indicator_color6 Gray
#property indicator_color5 Gray
#property indicator_color4 Gray
#property indicator_color3 Gray
#property indicator_color2 Gray
#property indicator_color1 Gray

extern int step       = 2;
extern int kPeriod1   = 13;
extern int slowing1   = 3;
extern int normLimit  = 26;
extern int timeframe  = 5;


#define STACK_SIZE 4

int kPeriod[STACK_SIZE];
int dPeriod[STACK_SIZE];
int slowing[STACK_SIZE];

double lineBuffer0[];
double lineBuffer1[];
double lineBuffer2[];
double lineBuffer3[];
double lineBuffer4[];
double lineBuffer5[];
double lineBuffer6[];
double lineBuffer7[];
int init() {
    IndicatorBuffers(8);
    if(timeframe < Period()) {
        timeframe = Period();
    }
    kPeriod[0] = kPeriod1;
    dPeriod[0] = slowing1;
    slowing[0] = slowing1;
    
    int i;
    for(i = 1;i < STACK_SIZE; i++) {
        kPeriod[i] = kPeriod1 + step * i;
        dPeriod[i] = kPeriod1 + step * i;
        slowing[i] = slowing1 + step * i;
    }

    string indicatorName = "StochasticStack_v1.3(" + kPeriod[0];
    for(i = 1; i < STACK_SIZE; i++) {
        indicatorName = indicatorName + "," +  kPeriod[i];
    }
    indicatorName = indicatorName + " " + switch.getTimeframeName(timeframe) +  " )";
    IndicatorShortName(indicatorName);
    
    for(i = 0;i < STACK_SIZE; i++) {
        SetIndexLabel(i, "kPeriod " + kPeriod[i]);
        SetIndexDrawBegin(i, kPeriod[i]);
    }
    
    SetIndexBuffer(0, lineBuffer0);
    SetIndexBuffer(1, lineBuffer1);
    SetIndexBuffer(2, lineBuffer2);
    SetIndexBuffer(3, lineBuffer3);
    SetIndexBuffer(4, lineBuffer4);
    SetIndexBuffer(5, lineBuffer5);
    SetIndexBuffer(6, lineBuffer6);
    SetIndexBuffer(7, lineBuffer7);  
    
    if(timeframe < Period()) {
        timeframe = Period();
    }
}

int i;
int start(){
    int indicatorCounted = IndicatorCounted();
    if (indicatorCounted < 0) { 
        return (-1);
    }
    if(indicatorCounted > 0) {
       indicatorCounted--;
    }
    int limit = Bars - indicatorCounted - 1;
    int tfDiv = timeframe / Period();
    limit = MathMax(limit, tfDiv);
    
    for(i = limit; i >= 0; i--) {
        lineBuffer4[i] = iStochastic(NULL, timeframe, kPeriod[0], dPeriod[0], slowing[0], MODE_SMA, 1, 0, i / tfDiv);
        lineBuffer5[i] = iStochastic(NULL, timeframe, kPeriod[1], dPeriod[1], slowing[1], MODE_SMA, 1, 0, i / tfDiv);
        lineBuffer6[i] = iStochastic(NULL, timeframe, kPeriod[2], dPeriod[2], slowing[2], MODE_SMA, 1, 0, i / tfDiv);
        lineBuffer7[i] = iStochastic(NULL, timeframe, kPeriod[3], dPeriod[3], slowing[3], MODE_SMA, 1, 0, i / tfDiv);
    }
    
    for(i = limit; i >= 0; i--) {
        lineBuffer0[i] = indiUtils.normValue(lineBuffer4, normLimit, i);
        lineBuffer1[i] = indiUtils.normValue(lineBuffer5, normLimit, i);
        lineBuffer2[i] = indiUtils.normValue(lineBuffer6, normLimit, i);
        lineBuffer3[i] = indiUtils.normValue(lineBuffer7, normLimit, i);
    }
}

double indiUtils.normValue(double &buffer[], int normLimit, int i) {
   double max = buffer[ArrayMaximum(buffer, normLimit, i)];
   double min = buffer[ArrayMinimum(buffer, normLimit, i)];
   double value = 0;
   if(max - min == 0) {
      value = 1;
   } else {
      value = (buffer[i] - min) / (max - min);
   }
   if (value > 1) {
      value = 1;
   } else if(value < 0) {
      value = 0;
   }
   return(value);
}

bool switch.isValidTF(int period) {
   switch(period){
        case PERIOD_MN1:
            return(true);
        case PERIOD_W1:
            return(true);
        case PERIOD_D1:
            return(true);
        case PERIOD_H4:
            return(true);    
        case PERIOD_H1:
            return(true);
        case PERIOD_M30:
            return(true);
        case PERIOD_M15:
            return(true);
        case PERIOD_M5:
            return(true);
        case PERIOD_M1:
            return(true);
        default: 
            return(false);
    }
}


string switch.getTimeframeName(int period) {
    switch(period){
        case PERIOD_MN1:
            return("MN1");
        case PERIOD_W1:
            return("W1");
        case PERIOD_D1:
            return("D1");
        case PERIOD_H4:
            return("H4");     
        case PERIOD_H1:
            return("H1");
        case PERIOD_M30:
            return("M30");
        case PERIOD_M15:
            return("M15");
        case PERIOD_M5:
            return("M5");
        case PERIOD_M1:
            return("M1");
        default: 
            return(switch.getTimeframeName(Period()));
    }
}